<?php
/*
Plugin Name: Meta Functions Shortcode
Plugin URI: http://www.matthiasamacher.ch/wordpress/meta-functions-shortcode
Description: This plugin adds a shortcode of the form [meta func="" name="" alt=""]. Where {name} is the name of a meta tag in your article and {func} is the function to perform on it either (url, img or plain). {alt} is either the caption text of a URL or returned, if the meta is empty or the ALT tag of the image.
Author: Matthias Amacher
Version: 1.0
Author URI: http://www.matthiasamacher.ch
*/
/*  Copyright 2009 Matthias Amacher, Switzerland

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

function meta_functions_shortcode($atts) {
	global $post;
	extract(shortcode_atts(array (
		'func' => 'plain',
		'name' => '',
		'alt' => '',
		
	), $atts));

	
	if ($name == "") {
		return mfs_docustom($atts);
		
	}
	$meta = get_post_meta($post->ID, $name, true);

	$meta_function = "meta_functions_shortcode_$func";
	
	if (function_exists($meta_function)) {
		return $meta_function ($meta, $alt);

	} 
	
	return mfs_docustom($atts);
		
	
}
function mfs_separate_by($string,$char=","){
	$result = array();
	$str = "";
	for ($i = 0;$i<strlen($string);$i++){
		$c = substr($string,$i,1);
		if ($c==$char){
			$result[] = $str;
			$str = "";
		} else {
			$str = $str . $c;
		}
	}
	$result[] = $str;
	return $result;

}

function mfs_docustom($atts){
	global $post;
	extract($atts);
	if ($func==""){
		return "";
	}
	$opts = get_option('meta_functions_shortcode');
	if (empty($opts[$func])){
		return "(mfs: undefined function)";
	}
	
	$o = $opts[$func];
	
	$args = mfs_separate_by($o['mfs_function_args'],",");
	$result = stripslashes($o['mfs_function_result']);
	
	$args_values = array();
	
	foreach($args as $arg){
		$meta = get_post_meta($post->ID, $atts[$arg], true);
		$a = $atts[$arg];
	
		$result = str_replace('{'.$arg.'}',$meta,$result);
		
	}
	
	return $result;
	
	
	
}

function meta_functions_shortcode_img($meta, $alt = "") {
	if ($meta != "") {
		if ($alt != "") {
			return "<img src='$meta' alt='$alt'/>";
		} else {
			return "<img src='$meta'/>";
		}
	}
	return "";
}

function meta_functions_shortcode_plain($meta, $alt = "") {
	if ($meta != "") {
		return $meta;
	} else {
		return $alt;
	}
}
function meta_functions_shortcode_url($meta, $alt = "") {
	if ($meta != "") {
		if ($alt != "") {
			return "<a href='$meta'>$alt</a>";
		} else {
			return "<a href='$meta'>$meta</a>";
		}
	}
	return "";
}


/*
// Extending the Plugin:
function meta_functions_shortcode_{func}($meta, $alt="") {
       return "your desired text or generated html content";
}*/

add_shortcode('meta', 'meta_functions_shortcode');

add_action('admin_menu', 'mfs_add_options');

function mfs_add_options() {
	add_options_page('Meta Functions Shortcode', 'Meta Functions Shortcode', 8, __FILE__, 'mfs_show_options');

}

function mfs_show_options() {
	include ('mfs_options_page.php');
}


add_action('wp_ajax_showmfsfunction', 'mfs_do_show_function');
add_action('wp_ajax_deletemfsfunction', 'mfs_do_delete_function');

function mfs_do_show_function() {
	check_ajax_referer("meta_functions_shortcode");
	$opts = get_option('meta_functions_shortcode');
	$o = $opts[$_POST['func_name']];
	$o['mfs_function_result'] = stripslashes($o['mfs_function_result']);
	echo json_encode($o);

	die();
}

function mfs_do_delete_function() {
	check_ajax_referer("meta_functions_shortcode");
	$opts = get_option('meta_functions_shortcode');

	unset ($opts[$_POST['func_name']]);
	update_option('meta_functions_shortcode', $opts);
	$opts = get_option('meta_functions_shortcode');

	foreach ($opts as $key => $val) {
		echo json_encode($opts[$key]);
		die();
	}

	die();
}
?>